// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_continuous_deployment/src/core/commands/build_app/commands/build_app_bundle/build_app_bundle_task.dart';
import 'package:wyatt_continuous_deployment/src/core/commands/deploy_app/commands/deploy_android/deploy_android_task.dart';
import 'package:wyatt_continuous_deployment/src/core/injection/injectable.dart';
import 'package:wyatt_continuous_deployment/src/core/task/task_cubit.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/context.dart';

class BuildAndDeployAndroidAppTask extends TaskCubit {
  factory BuildAndDeployAndroidAppTask() => BuildAndDeployAndroidAppTask._(
        checkToolsUsecase: getIt(),
        checkConfigFileUsecase: getIt(),
      );

  BuildAndDeployAndroidAppTask._({
    required super.checkToolsUsecase,
    required super.checkConfigFileUsecase,
  });

  @override
  Future<void> onRun(Context ctx) async {
    // 1. Build Android app
    final buildAppBundleTask = BuildAppBundleTask();
    await buildAppBundleTask.onRun(ctx);
    if (buildAppBundleTask.status != 0) {
      return emit(buildAppBundleTask.state);
    }

    // 2. Deploy it
    final deployAndroidTask = DeployAndroidTask();
    await deployAndroidTask.onRun(ctx);
    if (deployAndroidTask.status != 0) {
      return emit(deployAndroidTask.state);
    }
  }
}
