// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

class ModelIdentifier<Model> {
  const ModelIdentifier({
    required this.getIdentifier,
  });

  final String Function(Model model) getIdentifier;

  String? getIdentifierOrNull(Model? model) {
    if (model == null) {
      return null;
    }

    return getIdentifier(model);
  }

  bool isIdentifier({
    required Model? model,
    required String identifier,
  }) {
    if (model == null) {
      return false;
    }

    return getIdentifier(model) == identifier;
  }

  bool areEqual(
    Model? model1,
    Model? model2,
  ) {
    if (model1 == null && model2 == null) {
      return true;
    }

    if (model1 == null || model2 == null) {
      return false;
    }

    return getIdentifier(model1) == getIdentifier(model2);
  }
}
