// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_crud_bloc/src/core/model_mapper.dart';
import 'package:wyatt_crud_bloc/src/domain/entities/query.dart';

/// {@template crud_repository}
/// An abstract class that represents a SCRUD repository.
/// {@endtemplate}
abstract class CrudRepository<Model> extends BaseRepository {
  /// {@macro crud_repository}
  const CrudRepository();

  ModelMapper<Model> get modelMapper;

  /// Creates a new object.
  FutureOrResult<void> create(Model object, {String? id});

  /// Gets an object by its [id].
  FutureOrResult<Model?> get(String id);

  /// Gets all objects.
  FutureOrResult<List<Model?>> getAll();

  /// Updates an object by its [id].
  FutureOrResult<void> update(
    String id, {
    Model? object,
    Map<String, dynamic>? raw,
  });

  /// Updates all objects.
  FutureOrResult<void> updateAll(Map<String, Object?> raw);

  /// Deletes an object by its [id].
  FutureOrResult<void> delete(String id);

  /// Deletes all objects.
  FutureOrResult<void> deleteAll();

  /// Queries objects by [conditions].
  FutureOrResult<List<Model?>> search(List<Query> conditions);

  /// Streams objects by [conditions].
  StreamResult<List<Model?>> stream({
    String? id,
    List<Query>? conditions,
  });
}
